*!*	/////////////////////////////////////////////////////////////////////////////
*!*	//
*!*	//  FastRepF.PRG
*!*	//
*!*	//  Copyright:		
*!*	//       Spirin Sergey, (c) 2006-2011. All rights reserved.
*!*	//  
*!*	//  Contents:
*!*	//       Source File for frReportManager and other classes for 
*!*	//       "FastReport for FoxPro" (FrFox)
*!*	//   
*!*	///////////////////////////////////////////////////////////////////////////////


#define cReport         "Report"
#define cPreviewOptions "Report.PreviewOptions"
#define cPrintOptions   "Report.PrintOptions"
#define cReportOptions  "Report.ReportOptions"
#define cEngineOptions  "Report.EngineOptions"


Define Class frReportManager as Custom
   DIMENSION afrInst[1]
   HIDDEN FrFllLib
   
   ADD OBJECT PreviewOptions AS frPreviewOptions 
   ADD OBJECT PrintOptions AS frPrintOptions 
   ADD OBJECT ReportOptions AS frReportOptions
   ADD OBJECT EngineOptions AS frEngineOptions    
      
   PROCEDURE Init       
   LPARAMETERS cFrFllLib, cFrDllLib         
      IF PCOUNT( ) < 2 
         m.cFrDllLib = "FrSystF.dll"
         IF PCOUNT( ) < 1
            m.cFrFllLib = "fr.fll"
         ENDIF
      ENDIF
      
      IF AINSTANCE(afrInst, This.Class) = 0                  
         SET LIBRARY TO (m.cFrFllLib) ADDITIVE
         _fr_InitSyst(m.cFrDllLib)        
      ENDIF
      This.FrFllLib = m.cFrFllLib 
   ENDPROC

   PROCEDURE Destroy     
      IF (AINSTANCE(afrInst, This.Class) = 0)
         RELEASE LIBRARY (This.FrFllLib)
      ENDIF
   ENDPROC
   
   
*!*	///////////////////////////////////////////////////////////////////////
*!*	/  DesignReport() - Calls the report designer
*!*	///////////////////////////////////////////////////////////////////////
      
   PROCEDURE DesignReport
     _fr_DesignReport()
   ENDPROC
   
*!*	////////////////////////////////////////////////////////////////////////////
*!*	//  SetWorkArea() - Add WorkArea or modify params of WorkArea. 
*!*	//                  cFrAlias     - any string that will represent WorkArea in FR
*!*	//                  nWorkArea    - number of WorkArea
*!*	//                  lOem         - data in oem codepage (default - .f.)
*!*	//                  nRangeParams - Range Params with 3 elements:
*!*	//                    1 element - FR_RB_FIRST(0) or FR_RB_CURRENT(1)
*!*	//                    2 element - FR_RE_LAST(0) or FR_RE_CURRENT(1) or FR_RE_COUNT(2)
*!*	//                    3 element - A number of records in the WorkArea, 
*!*	//                                if the second element = FR_RE_COUNT.
*!*	//                      Default - {FR_RB_FIRST, FR_RE_LAST, 0}
*!*	////////////////////////////////////////////////////////////////////////////

   FUNCTION SetWorkArea      
      LPARAMETERS cFrAlias, nWorkArea, lOem, nRangeParam1, nRangeParam2, nRangeParam3
   RETURN _fr_SetWorkArea(m.cFrAlias, m.nWorkArea, m.lOem, m.nRangeParam1, m.nRangeParam2, m.nRangeParam3)
   
*!*	////////////////////////////////////////////////////////////////////////
*!* //   SetDataSessionID() - Set current DataSessionID for FrFox  
*!*	////////////////////////////////////////////////////////////////////////   
   
   PROCEDURE SetDataSessionID(nDataSessionID)
     _fr_SetDataSessionID(m.nDataSessionID)
   ENDPROC


*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SetMasterDetail() - Creating master/detail relationships
*!*	//                      between two WorkAreas (Delphi realization based on
*!*	//                      "SET KEY" - FoxPro function).
*!*	//
*!*	//                      You can use it instead "SET RELATION" in FoxPro code.
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION SetMasterDetail(cMasterAlias, cDetailAlias, cExpression)
   RETURN _fr_SetMasterDetail(m.cMasterAlias, m.cDetailAlias, m.cExpression)

*!*	////////////////////////////////////////////////////////////////////////////
*!*	//  SetResyncPair() - When DbRelation-relationships is set in XBase++
*!*	//                    code, you need notify FastReport about it.
*!*	//                    
*!*	//                    It means that after  changing of parent workarea cursor 
*!*	//                    position, child dataset resynchronize its cursor position 
*!*	//                    with XBase++ data.
*!*	////////////////////////////////////////////////////////////////////////////

   FUNCTION SetResyncPair(cParentAlias, cChildlAlias)
   RETURN _fr_SetResyncPair(m.cParentAlias, m.cChildlAlias) 

*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SetUserDataSet() - Add or modify UserDataSet. UserDataSet allows 
*!*	//                     constructing reports, which are not connected 
*!*	//                     to the data from DB, but do receive data from 
*!*	//                     other sources (for example, array, file, etc.). 
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION SetUserDataSet(cFrAlias, cFields, cGoTop, cSkipPlus1, cSkipMinus1, cCheckEOF, cGetValue)
   RETURN _fr_SetUserDataSet(m.cFrAlias, m.cFields, m.cGoTop, m.cSkipPlus1, m.cSkipMinus1, m.cCheckEOF, m.cGetValue)

*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SetTxtDataSet() - Add or modify TxtDataSet. TxtDataSet allows 
*!*	//                    work with big txt-files by lines.
*!*	////////////////////////////////////////////////////////////////////////


   FUNCTION SetTxtDataSet(cFrAlias, cFileName)
   RETURN _fr_SetTxtDataSet(m.cFrAlias, m.cFileName)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  Resync() - method for resynchronize the dataset with underlying 
*!*	//             physical data when making FoxPro calls that may change 
*!*	//             the workarea cursor position.            
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION Resync(cFrAlias)
   RETURN _fr_Resync(m.cFrAlias)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  RemoveDataSet() - Remove WorkArea or UserDataSet from Fast Report 
*!*	//                    data list.
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION RemoveDataSet(cFrAlias)
   RETURN _fr_RemoveDataSet(m.cFrAlias)

*!*	////////////////////////////////////////////////////////////////////////
*!*	//  ClearDataSets() - Remove All WorkAreas and UserDataSets from Fast 
*!*	//                    Report data list.
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE ClearDataSets()
      _fr_ClearDataSets()
   ENDPROC 
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  ClearMasterDetail() - Remove  master/detail relationships
*!*	//                        between two WorkAreas 
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE ClearMasterDetail(cDetailAlias)
      _fr_ClearMasterDetail(m.cDetailAlias) 
   ENDPROC    
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  ClearResyncPair() -   Remove  "resync" relationships
*!*	//                        between two WorkAreas 
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION ClearResyncPair(cParentAlias, cChildlAlias)
   RETURN _fr_ClearResyncPair(m.cParentAlias, m.cChildlAlias)
      
*!*	///////////////////////////////////////////////////////////////////////////////////////
*!*	//  SetFieldAliases() - Set symbolic names of data set fields and 
*!*	//                      show/hide some fields. For example:
*!*	//         SetFieldAliases("Customers", "CUSTNO=Cust No;Company;ADDR1=Address;Phone")
*!*	//
*!*	//         Only 4 fields will be visible, CUSTNO and ADDR1 will be visible as 'Cust No'
*!*	//         and 'Address'
*!*	///////////////////////////////////////////////////////////////////////////////////////

   FUNCTION SetFieldAliases(cFrAlias, cFieldAliases)
   RETURN _fr_SetFieldAliases(m.cFrAlias, m.cFieldAliases)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SetADOConnectStr() - Set ConnectionString for ADO-DataBase object 
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION SetADOConnectStr(cFrAlias, cStr)
   RETURN _fr_SetADOConnectStr(m.cFrAlias, m.cStr)
   
*!*	/////////////////////////////////////////////////////////////////////////////////
*!*	//  SetADOConnectPartStr() - Set only one part of ConnectionString 
*!*	//                           for ADO-DataBase object. For example:
*!*	//     SetADOConnectPartStr("ADODatabase1", "Data Source", SomeFile)
*!*	/////////////////////////////////////////////////////////////////////////////////

   FUNCTION SetADOConnectPartStr(cFrAlias, cKey, cValue)
   RETURN _fr_SetADOConnectPartStr(m.cFrAlias, m.cKey, m.cValue)
   

                  
*!*	//////////////////////////////////////////////////////////////////////////
*!*	//  ShowReport() - Starts a report and displays a result in the preview
*!*	//                 window. If the lNotClearLastReports parameter is True
*!*	//                 then a report will be added
*!*	//                 to the previously constructed one, otherwise the
*!*	//                 previously constructed report is cleared (by default).
*!*	//////////////////////////////////////////////////////////////////////////

   PROCEDURE ShowReport
      LPARAMETERS lNotClearLastReports
      _fr_ShowReport(m.lNotClearLastReports)
   ENDPROC
   
*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  PrepareReport() - Starts a report without a preview window. 
*!*	//                    nNotClear - the same as in ShowReport()
*!*	///////////////////////////////////////////////////////////////////////////

   FUNCTION PrepareReport
      LPARAMETERS lNotClearLastReports
   RETURN _fr_PrepareReport(m.lNotClearLastReports)
   
*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  ShowPreparedReport() - Displays the report, which was previously built 
*!*	//                         via the "PrepareReport" call.                     
*!*	///////////////////////////////////////////////////////////////////////////

   PROCEDURE ShowPreparedReport
      _fr_ShowPreparedReport()
   ENDPROC
           
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  LoadFromFile() - Loads a report from a file with given name. 
*!*	//                   Returns .t. if the file is loaded successfully
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION LoadFromFile
      LPARAMETERS cFileName      
   RETURN _fr_LoadFromFile(m.cFileName)
   
*!*	////////////////////////////////////////////////////////////////////////////
*!*	//  Print() - Prints a report. See frPrintOptions class.
*!*	//            lDialogIsChild - if true then PrintDialog will be the child of
*!*	//                             current foreground window.
*!*	//                             Default -False
*!*	////////////////////////////////////////////////////////////////////////////

   FUNCTION Print(lDialogIsChild)
   RETURN _fr_Print(m.lDialogIsChild)   

*!*	////////////////////////////////////////////////////////////////////////
*!*	// DoExport() - Exports a report using the specified export filter 
*!*	//              object. Possible values of Param:
*!*	//
*!*	//                  PDFExport, HTMLExport, RTFExport, CSVExport,
*!*	//                  XLSExport, DotMatrixExport, BMPExport, JPEGExport,
*!*	//                  TIFFExport, GIFExport, SimpleTextExport, MailExport,
*!*	//                  XMLExport, TXTExport
*!*	//
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE DoExport(cExportObjectName)
     _fr_DoExport(m.cExportObjectName)
   ENDPROC

   
*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  SetIcon() -  the numeric resource ID or string resource name or file 
*!*	//               of the icon of FastReport-windows
*!*	///////////////////////////////////////////////////////////////////////////

   FUNCTION SetIcon
      LPARAM xValue, lFromFile
      LOCAL lRes
      IF VARTYPE(xValue) = "C"
         lRes = _fr_SetIcon(0, m.xValue, m.lFromFile) 
      ELSE
         lRes = _fr_SetIcon(m.xValue, "")
      ENDIF
   RETURN lRes
   
*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  SetTitle() -  Title of taskbar FastReport-window
*!*	///////////////////////////////////////////////////////////////////////////

   PROCEDURE SetTitle(cTitle)
      _fr_SetTitle(m.cTitle)  
   ENDPROC 
   
*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  LoadLangRes() - Load language resource file
*!*	///////////////////////////////////////////////////////////////////////////

   FUNCTION LoadLangRes(cLangFile)
   RETURN _fr_LoadLangRes(m.cLangFile)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  Clear() - Clears a report. 
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE Clear
     _fr_Clear()
   ENDPROC

*!*	////////////////////////////////////////////////////////////////////////
*!*	// GetProperty()/  Universal functions that set or return any property 
*!*	// SetProperty() - (except object property) of any Fast Report object. 
*!*	//                   Possible values of cObjectName:
*!*	//
*!*	//                  Report, Designer,
*!*	//                  PDFExport, HTMLExport, RTFExport, CSVExport,
*!*	//                  XLSExport, DotMatrixExport, BMPExport, JPEGExport,
*!*	//                  TIFFExport, GIFExport, SimpleTextExport, MailExport
*!*	//                  or any object name in current report.
*!*	//
*!*	//                  For properties of objects that is property of this
*!*	//                  "main" objects '.'-delimeter must be used. For
*!*	//                  example:
*!*	//                    "Designer.DefaultFont"
*!*	//
*!*	//                  For full list of values for cPropName param see Fast
*!*	//                  Report documentation:
*!*	//                   http://www.fast-report.com/en/documentation/
*!*	// 
*!*	//                  Examples:
*!*	//                    SetProperty("Report", "ScriptLanguage", "PascalScript")
*!*	//                    SetProperty("PDFExport", "FileName", "My file.pdf")
*!*	//                    SetProperty("PDFExport", "ShowDialog", .f.)
*!*	//
*!*	//                  For Delphi enumerated types and set type use its
*!*	//                  string representation, for example:
*!*	//                    SetProperty("Designer.DefaultFont", "Style", "[fsBold, fsItalic]")
*!*	//
*!*	//                   To set object property use SetObjProperty()
*!*	//                 
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION GetProperty(cObjectName, cPropName, lIsDateTime)
   RETURN _fr_GetProperty(m.cObjectName, m.cPropName, m.lIsDateTime)

   FUNCTION SetProperty(cObjectName, cPropName, xValue)
   RETURN _fr_SetProperty(m.cObjectName, m.cPropName, m.xValue)
   
   FUNCTION SetObjProperty(cObjectName, cPropName, cObjToSetName)
   RETURN _fr_SetObjProperty(m.cObjectName, m.cPropName, m.cObjToSetName)


*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SetEventHandler() - Set Event Handler to FastReports event. 
*!*	//                      Possible values of params:
*!*	//          cObjectName - Report:
*!*	//                         cEventName: OnAfterPrint(cObjName), 
*!*	//                                     OnAfterPrintReport(), 
*!*	//                                     OnBeforeConnect(cConnectName), 
*!*	//                                     OnBeforePrint(cObjName), 
*!*	//                                     OnBeginDoc(), 
*!*	//                                     OnClickObject(cObjName, nButton), 
*!*	//                                     OnEndDoc(), 
*!*	//                                     OnGetValue(cVarName) -> VarValue ,
*!*	//                                     OnMouseOverObject(cObjName),
*!*	//                                     OnPreview(),
*!*	//                                     OnPrintPage(nCopyNo), 
*!*	//                                     OnPrintReport(),
*!*	//                                     OnProgress(nProgressType, nProgress),
*!*	//                                     OnProgressStop(nProgressType, nProgress),
*!*	//                                     OnProgressStart(nProgressType, nProgress),
*!*	//                                     OnUserFunction(cMethodName, Params...) -> xResult 
*!*	//
*!*	//         cObjectName - Designer:
*!*	//                         cEventName: OnInsertObject(),
*!*	//                                     OnLoadReport(),
*!*	//                                     OnSaveReport(lSaveAs),
*!*	//                                     OnShow(),
*!*	//                                     OnShowStartupScreen()
*!*	//
*!*	//         Example:
*!*	//            SetEventHandler("Report", "OnProgress", "MyObj.MyShowProgress"})
*!*	//
*!*	//         To clear EventHandler pass "" as cEventHandler param
*!*	//
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION SetEventHandler(cObjectName, cEventName, cEventHandler)
   RETURN _fr_SetEventHandler(m.cObjectName, m.cEventName, m.cEventHandler)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  AddFunction() - Adds a user function to the list of the functions 
*!*	//                  available in the report. OnUserFunction-event handler 
*!*	//                  must be assigned. 
*!*	//                    cFunction - function declaration in Pascal syntax.
*!*	//                    
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION AddFunction(cFunction, cCategory, cDescription)
      _fr_AddFunction(m.cFunction, m.cCategory, m.cDescription)
   ENDPROC 
   
*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  AddVariable() - Adds a user variable to the list of the variables
*!*	//                  available in the report. 
*!*	//
*!*	//    IMPOTANT:  When accessing a report variable its value is calculated 
*!*	//               if it is of string type. That means the variable which 
*!*	//               value is "GetXppVar(SomeVar)"  will return a value of 
*!*	//               SomeVar - variable, but not the "GetFoxVar(SomeVar)" string. 
*!*	//               You should be careful when assigning a string-type values 
*!*	//               to report variables. Simple string value must be in 
*!*	//               Pascal string quotes - '
*!*	//                Examples for xValue:
*!*	//                  "GetFoxVar(SomeVar)"  -> Value of SomeVar
*!*	//                  "'GetFoxVar(SomeVar)'" -> string  
*!*	//
*!*	/////////////////////////////////////////////////////////////////////////

   PROCEDURE AddVariable(cCategory, cName, xValue)
      _fr_AddVariable(m.cCategory, m.cName, m.xValue)
   ENDPROC
   
*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  SetVariable() - Modify a user variables value
*!*	/////////////////////////////////////////////////////////////////////////

   PROCEDURE SetVariable(cName, xValue)
      _fr_SetVariable(m.cName, m.xValue)
   ENDPROC
                     
*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  GetVariable() - Get a user variables value
*!*	/////////////////////////////////////////////////////////////////////////

   FUNCTION GetVariable(cName)         
   RETURN _fr_GetVariable(m.cName)
   
*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  DeleteVariable() - Delete a user variable
*!*	/////////////////////////////////////////////////////////////////////////
            
   PROCEDURE DeleteVariable(cName)
      _fr_DeleteVariable(m.cName)
   ENDPROC 

*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  DeletCategory() - delete a category with ALL its variables.
*!*	/////////////////////////////////////////////////////////////////////////
      
   PROCEDURE DeleteCategory(cCategory)
      _fr_DeleteCategory(m.cCategory)
   ENDPROC
   
*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  SetScriptVar()/ - Get or Set script variable
*!*	//  GetScriptVar
*!*	//                  Instead of report variables, script variables are 
*!*	//                  not visible in the report variables list and is not 
*!*	//                  calculated. You can define them using FastScript 
*!*	//                  methods
*!*	/////////////////////////////////////////////////////////////////////////
      
   PROCEDURE SetScriptVar(cName, xValue)
      _fr_SetScriptVar(m.cName, m.xValue)
   ENDPROC 
            
   FUNCTION GetScriptVar(cName)   
   RETURN _fr_GetScriptVar(m.cName)

*!*	/////////////////////////////////////////////////////////////////////////
*!*	//  Calc() - Calculates the any FastScript expression and returns the 
*!*	//           result.
*!*	/////////////////////////////////////////////////////////////////////////
      
   PROCEDURE Calc(cExpression)
   RETURN _fr_Calc(m.cExpression)      
                                             
*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  SetFormatSettings() - Set Settings for Format functions in FrFox
*!*	///////////////////////////////////////////////////////////////////////////

   PROCEDURE SetFormatSettings(nSetting, xValue)
      _fr_SetFormatSettings(m.nSetting, m.xValue)
   ENDPROC
  
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  LoadFromResource() - Loads a report from a resource with given name or number. 
*!*	//                       Returns .t. if the report is loaded successfully
*!*	////////////////////////////////////////////////////////////////////////
  
   FUNCTION LoadFromResource(xValue)
   LOCAL lRes
      IF VarType(m.xValue) = "C"
         m.lRes = _fr_LoadFromResource(0, m.xValue)
      ELSE
         m.lRes = _fr_LoadFromResource(m.xValue)
      ENDIF
   RETURN m.lRes
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  LoadFromBlob() - Loads a report from a memo or blob field. 
*!*	//                   Returns .t. if the file is loaded successfully
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION LoadFromBlob(nWorkArea, cFieldName)
   RETURN _fr_LoadFromBlob(m.nWorkArea, m.cFieldName)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  LoadFromString() - Loads a report from a character string. 
*!*	//                     Returns .t. if the string is loaded successfully
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION LoadFromString(cStringVal)
   RETURN _fr_LoadFromString(m.cStringVal)
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SaveToFile() - Saves a report to a file with given name. 
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE SaveToFile(cFileName)
      _fr_SaveToFile(m.cFileName)
   ENDPROC
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SaveToBlob() - Saves a report to a memo or blob field.             
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE SaveToBlob(nWorkArea, cFieldName)
      REPLACE (m.cFieldName) WITH _fr_SaveToString() IN (m.nWorkArea)
     * _fr_SaveToBlob(m.nWorkArea, m.cFieldName)
   ENDPROC
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  SaveToString() - Saves a report to a character string.             
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION SaveToString()
   RETURN _fr_SaveToString()
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  Multiple reports section 
*!*	//  AddReport() - adds new report and return its number (like with workarea)
*!*	//  SelectReport() - select existing report (like with workarea)
*!*	//  RemoveReport() - removes existing report from memory
*!*	//  ClearReports() - removes all existing reports from memory (except with number 0)
*!*	////////////////////////////////////////////////////////////////////////
   
   FUNCTION AddReport()
   RETURN _fr_AddReport()
   
   FUNCTION SelectReport(nReport)
   RETURN _fr_SelectReport(m.nReport)

   PROCEDURE RemoveReport(nReport)
      _fr_RemoveReport(m.nReport)
   ENDPROC

   PROCEDURE ClearReports()
      _fr_ClearReports()
   ENDPROC
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  LoadFPFile() -     Loads a prepared report from a saved FP3-file. 
*!*	//                     Returns .t. if the file is loaded successfully
*!*	////////////////////////////////////////////////////////////////////////

   FUNCTION LoadFPFile(cFileName)
   RETURN _fr_LoadFPFile(m.cFileName)


*!*	///////////////////////////////////////////////////////////////////////////
*!*	//  SaveToFPFile() - Saves a prepared report to a FP3-file with given name. 
*!*	///////////////////////////////////////////////////////////////////////////

   PROCEDURE SaveToFPFile(cFileName)
      _fr_SaveToFPFile(m.cFileName)
   ENDPROC
   
*!*	////////////////////////////////////////////////////////////////////////
*!*	//  LoadStyleSheet() - Loads Style Sheet from file
*!*	//  LoadStyleSheetFromString() - Loads Style Sheet from string
*!*	////////////////////////////////////////////////////////////////////////

   PROCEDURE LoadStyleSheet(cFileName)
      _fr_LoadStyleSheet(m.cFileName)
   ENDPROC

   PROCEDURE LoadStyleSheetFromString(cString)
      _fr_LoadStyleSheetFromString(m.cString)
   ENDPROC
   
*!*	/////////////////////////////////////////////////////////////////////////////////
*!*	//
*!*	// ManualBuild - section
*!*	//
*!*	/////////////////////////////////////////////////////////////////////////////////

   PROCEDURE StartManualBuild(cManualBlock, nOrientation, nPaperSize, nUnits)   
      _fr_StartManualBuild(m.cManualBlock, m.nOrientation, m.nPaperSize, m.nUnits)
   ENDPROC 
   
   FUNCTION MemoAt(cStr, nLeft, nTop, nWidth, nHeight)
   RETURN _fr_MemoAt(m.cStr, m.nLeft, m.nTop, m.nWidth, m.nHeight)

   FUNCTION LineAt(nLeft, nTop, nWidth, nHeight)
   RETURN _fr_LineAt(m.nLeft, m.nTop, m.nWidth, m.nHeight)
   
   FUNCTION PictureAt(cFileName, nLeft, nTop, nWidth, nHeight)
   RETURN _fr_PictureAt(m.cFileName, m.nLeft, m.nTop, m.nWidth, m.nHeight)

   PROCEDURE  SetDefaultFontProperty(cPropName, xValue)
      _fr_SetDefaultFontProperty(m.cPropName, m.xValue)
   ENDPROC 
   
   PROCEDURE NewPage()
      _fr_NewPage()
   ENDPROC
   
   FUNCTION SetManualObjProperty(cObjName, cPropName, xValue)        
   RETURN _fr_SetManualObjProperty(m.cObjName, m.cPropName, m.xValue)   
   
*!*	/////////////////////////////////////////////////////////////////////////////////
*!*	//
*!*	// Manual report creation section
*!*	//
*!*	/////////////////////////////////////////////////////////////////////////////////

   PROCEDURE NewReport(cPageName)
      _fr_NewReport(m.cPageName)
   ENDPROC
   
   PROCEDURE AddPage(cPageName)
      _fr_AddPage(m.cPageName)
   ENDPROC
   
   PROCEDURE AddBand(cBandName, cParentPage, nBandType)
      _fr_AddBand(m.cBandName, m.cParentPage, m.nBandType)
   ENDPROC   
 
   FUNCTION AddGlobalDsToReport(cDataSetName)
   RETURN _fr_AddGlobalDsToReport(m.cDataSetName)

   FUNCTION  GetGlobalDSList()      
   RETURN _fr_GetGlobalDSList()
   
   FUNCTION AddMemo(cParent, cName, cStr, nLeft, nTop, nWidth, nHeight)
   RETURN _fr_AddMemo(m.cParent, m.cName, m.cStr, m.nLeft, m.nTop, m.nWidth, m.nHeight)
   
   FUNCTION AddLine(cParent, cName, nLeft, nTop, nWidth, nHeight)
   RETURN _fr_AddLine(m.cParent, m.cName, m.nLeft, m.nTop, m.nWidth, m.nHeight)
   
   FUNCTION  AddPicture(cParent, cName, cFileName, nLeft, nTop, nWidth, nHeight)
   RETURN _fr_AddPicture(m.cParent, m.cName, m.cFileName, m.nLeft, m.nTop, m.nWidth, m.nHeight)

   FUNCTION IsObjectExists(cObjName)
   RETURN _fr_IsObjectExists(m.cObjName)
    
   FUNCTION CreateFRObject(cClassName, cParent, cName, nLeft, nTop, nWidth, nHeight)
   RETURN _fr_CreateFRObject(m.cClassName, m.cParent, m.cName, m.nLeft, m.nTop, m.nWidth, m.nHeight)
   
   FUNCTION EnumAllObjects(cEnumExpression)
      _fr_EnumAllObjects(m.cEnumExpression)
   ENDPROC 

   FUNCTION GetPropList(cObjName)
   RETURN _fr_GetPropList(m.cObjName)
   
   PROCEDURE PrepareScript()
      _fr_PrepareScript()
   ENDPROC 

*!*	/////////////////////////////////////////////////////////////////////////////////
*!*	//
*!*	// tech methods
*!*	//
*!*	/////////////////////////////////////////////////////////////////////////////////

   FUNCTION frMessageBox(cText, cCaption, nFlag)
   RETURN _fr_frMessageBox(m.cText, m.cCaption, m.nFlag)
   
   FUNCTION frInputBox(cCaption, cPrompt, cDefault)
   RETURN _fr_frInputBox(m.cCaption, m.cPrompt, m.cDefault)

   FUNCTION SendMail(cServer, nPort, cUserField, cPasswordField,; 
       cFromField, cToField, cSubjectField, cCompanyField,;
       cTextField, cFileNames)
   RETURN _fr_SendMail(m.cServer, m.nPort, m.cUserField, m.cPasswordField,; 
       m.cFromField, m.cToField, m.cSubjectField, m.cCompanyField,;
       m.cTextField, m.cFileNames)
           
ENDDEFINE


*!*	//////////////////////////////////////////////////////////////////////
*!*	// frEngineOptions - A set of properties related to the FastReport 
*!*	//                   engine: 
*!*	//////////////////////////////////////////////////////////////////////


Define Class frEngineOptions as Custom
   ConvertNulls =.t.
   DoublePass = .f.
   MaxMemSize = 10
   PrintIfEmpty = .t.
   SilentMode = .f.
   * enum NewSilentMode = (simMessageBoxes, simSilent, simReThrow)
   NewSilentMode = ""
   TempDir = ""
   UseFileCache = .f.
   IgnoreDevByZero = .f.

   *----------------
   PROCEDURE ConvertNulls_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "ConvertNulls")
   ENDPROC
   
   PROCEDURE ConvertNulls_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cEngineOptions, "ConvertNulls", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE DoublePass_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "DoublePass")
   ENDPROC
   
   PROCEDURE DoublePass_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cEngineOptions, "DoublePass", m.lParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE MaxMemSize_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "MaxMemSize")
   ENDPROC
   
   PROCEDURE MaxMemSize_ASSIGN
      LPARAMETERS nParam
      _fr_SetProperty(cEngineOptions, "MaxMemSize", m.nParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE PrintIfEmpty_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "PrintIfEmpty")
   ENDPROC
   
   PROCEDURE PrintIfEmpty_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cEngineOptions, "PrintIfEmpty", m.lParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE SilentMode_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "SilentMode")
   ENDPROC
   
   PROCEDURE SilentMode_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cEngineOptions, "SilentMode", m.lParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE NewSilentMode_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "NewSilentMode")
   ENDPROC
   
   PROCEDURE NewSilentMode_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cEngineOptions, "NewSilentMode", m.cParam) 
   ENDPROC
   *----------------   
      
   PROCEDURE TempDir_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "TempDir")
   ENDPROC
   
   PROCEDURE TempDir_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cEngineOptions, "TempDir", m.cParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE UseFileCache_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "UseFileCache")
   ENDPROC
   
   PROCEDURE UseFileCache_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cEngineOptions, "UseFileCache", m.lParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE IgnoreDevByZero_ACCESS
      RETURN _fr_GetProperty(cEngineOptions, "IgnoreDevByZero")
   ENDPROC
   
   PROCEDURE IgnoreDevByZero_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cEngineOptions, "IgnoreDevByZero", m.lParam) 
   ENDPROC
   *----------------   

ENDDEFINE

*!*	//////////////////////////////////////////////////////////////////////
*!*	// frReportOptions - A set of properties, relating to the current 
*!*	//                     report 
*!*	//////////////////////////////////////////////////////////////////////


Define Class frReportOptions as Custom
   Author = ""
   Compressed =.f.
   CreateDate = .NULL.
   Description = ""
   InitString = ""
   cName = ""
   LastChange = .NULL.
   Password = ""
   VersionBuild = ""
   VersionMajor = ""
   VersionMinor = ""
   VersionRelease = ""
   
   *----------------
   PROCEDURE Author_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "Author")
   ENDPROC
   
   PROCEDURE Copies_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "Author", m.cParam) 
   ENDPROC
   *----------------

   PROCEDURE Compressed_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "Compressed")
   ENDPROC
   
   PROCEDURE Compressed_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cReportOptions, "Compressed", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE CreateDate_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "CreateDate", .t.)
   ENDPROC
   
   PROCEDURE CreateDate_ASSIGN
      LPARAMETERS dParam
      _fr_SetProperty(cReportOptions, "CreateDate", m.dParam) 
   ENDPROC
   *----------------
   
   PROCEDURE Description_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "Description.Text")
   ENDPROC
   
   PROCEDURE Description_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "Description.Text", m.cParam) 
   ENDPROC
   *----------------
   
   PROCEDURE InitString_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "InitString")
   ENDPROC
   
   PROCEDURE InitString_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "InitString", m.cParam) 
   ENDPROC
   *----------------

   PROCEDURE cName_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "Name")
   ENDPROC
   
   PROCEDURE cName_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "Name", m.cParam) 
   ENDPROC
   *----------------   
   
   PROCEDURE LastChange_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "LastChange", .t.)
   ENDPROC
   
   PROCEDURE LastChange_ASSIGN
      LPARAMETERS dParam
      _fr_SetProperty(cReportOptions, "LastChange", m.dParam) 
   ENDPROC
   *----------------   

   PROCEDURE Password_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "Password")
   ENDPROC
   
   PROCEDURE Password_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "Password", m.cParam) 
   ENDPROC
   *----------------   

   PROCEDURE VersionBuild_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "VersionBuild")
   ENDPROC
   
   PROCEDURE VersionBuild_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "VersionBuild", m.cParam) 
   ENDPROC
   *----------------   

   PROCEDURE VersionMajor_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "VersionMajor")
   ENDPROC
   
   PROCEDURE VersionMajor_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "VersionMajor", m.cParam) 
   ENDPROC
   *----------------      

   PROCEDURE VersionMinor_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "VersionMinor")
   ENDPROC
   
   PROCEDURE VersionMinor_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "VersionMinor", m.cParam) 
   ENDPROC
   *----------------         

   PROCEDURE VersionRelease_ACCESS
      RETURN _fr_GetProperty(cReportOptions, "VersionRelease")
   ENDPROC
   
   PROCEDURE VersionRelease_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cReportOptions, "VersionRelease", m.cParam) 
   ENDPROC
   *----------------         
   
ENDDEFINE



*!*	//////////////////////////////////////////////////////////////////////
*!*	// frPrintOptions - a set of properties, which relate to the report
*!*	//                  printing
*!*	//////////////////////////////////////////////////////////////////////

Define Class frPrintOptions as Custom
   Copies = 1
   Collate = .t.
   PageNumbers = ""
   Printer = ""
   PrintOnSheet = 0
   Reverse = .f.
   ShowDialog = .t.   
   * enum PrintMode = (pmDefault, pmSplit, pmJoin, pmScale)
   PrintMode = ""
   * enum PrintPages = (ppAll, ppOdd, ppEven)
   PrintPages = ""
   
   *----------------
   PROCEDURE Copies_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "Copies")
   ENDPROC
   
   PROCEDURE Copies_ASSIGN
      LPARAMETERS nParam
      _fr_SetProperty(cPrintOptions, "Copies", m.nParam) 
   ENDPROC
   *----------------
   
   PROCEDURE Collate_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "Collate")
   ENDPROC
   
   PROCEDURE Collate_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cPrintOptions, "Collate", m.lParam) 
   ENDPROC
   *----------------  

   PROCEDURE PageNumbers_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "PageNumbers")
   ENDPROC
   
   PROCEDURE PageNumbers_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cPrintOptions, "PageNumbers", m.cParam) 
   ENDPROC
   *----------------  

   PROCEDURE Printer_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "Printer")
   ENDPROC
   
   PROCEDURE Printer_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cPrintOptions, "Printer", m.cParam) 
   ENDPROC
   *---------------- 
   PROCEDURE PrintOnSheet_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "PrintOnSheet")
   ENDPROC
   
   PROCEDURE PrintOnSheet_ASSIGN
      LPARAMETERS nParam
      _fr_SetProperty(cPrintOptions, "PrintOnSheet", m.nParam) 
   ENDPROC
   *----------------  
   
   PROCEDURE Reverse_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "Reverse")
   ENDPROC
   
   PROCEDURE Reverse_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cPrintOptions, "Reverse", m.lParam) 
   ENDPROC
   *---------------- 
   
   PROCEDURE ShowDialog_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "ShowDialog")
   ENDPROC
   
   PROCEDURE ShowDialog_ASSIGN
      LPARAMETERS lParam
      _fr_SetProperty(cPrintOptions, "ShowDialog", m.lParam) 
   ENDPROC
   *----------------    

   PROCEDURE PrintMode_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "PrintMode")
   ENDPROC
   
   PROCEDURE PrintMode_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cPrintOptions, "PrintMode", m.cParam) 
   ENDPROC
   *----------------  

   PROCEDURE PrintPages_ACCESS
      RETURN _fr_GetProperty(cPrintOptions, "PrintPages")
   ENDPROC
   
   PROCEDURE PrintPages_ASSIGN
      LPARAMETERS cParam
      _fr_SetProperty(cPrintOptions, "PrintPages", m.cParam) 
   ENDPROC
   

ENDDEFINE

*!*	//////////////////////////////////////////////////////////////////////
*!*	// frPreviewOptions - A set of properties, relating to the report 
*!*	//                    preview
*!*	//////////////////////////////////////////////////////////////////////

Define Class frPreviewOptions as Custom
   AllowEdit = .t.
   DoubleBuffered =.t.
   Maximized =.t.
   Modal = .t.
   OutlineExpand = .t.
   OutlineVisible =.f.
   OutlineWidth = 120
   ThumbnailVisible = .f.
   PagesInCache = 50   
   PictureCacheInFile = .t.
   ShowCaptions = .f.
   Zoom = 1
   * enum ZoomMode = (zmDefault, zmWholePage, zmPageWidth, zmManyPages)
   ZoomMode = ""
   * set Buttons = (pbPrint, pbLoad, pbSave, pbExport, pbZoom, pbFind,
   *                pbOutline, pbPageSetup, pbTools, pbEdit, pbNavigator, pbExportQuick,
   *                pbNoClose, pbNoFullScreen, pbNoEmail) 
   Buttons = ""
   
*!*	   ////////////////////////////////////////////////////////////////////////
*!*	   //
*!*	   //  SetRemoveReportOnClose() - Set to remove(free) or not report at
*!*	   //                             preview closing. It does not work for 
*!*	   //                             report with number 0.
*!*	   //
*!*	   ////////////////////////////////////////////////////////////////////////


    PROCEDURE SetRemoveReportOnClose(lRemove)
       _fr_SetRemoveReportOnClose(m.lRemove)
    ENDPROC 
    
*!*	////////////////////////////////////////////////////////////////////////
*!*	// SetBounds() - Defines position and size of the preview window
*!*	////////////////////////////////////////////////////////////////////////

    PROCEDURE SetBounds(nLeft, nTop, nWidth, nHeight)
       _fr_SetBounds(m.nLeft, m.nTop, m.nWidth, m.nHeight)
    ENDPROC 
    
   
   *----------------
   PROCEDURE AllowEdit_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "AllowEdit")
   ENDPROC
   
   PROCEDURE AllowEdit_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "AllowEdit", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE DoubleBuffered_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "DoubleBuffered")
   ENDPROC
   
   PROCEDURE DoubleBuffered_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "DoubleBuffered", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE Maximized_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "Maximized")
   ENDPROC
   
   PROCEDURE Maximized_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "Maximized", m.lParam) 
   ENDPROC
   *----------------

   PROCEDURE Modal_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "Modal")
   ENDPROC
   
   PROCEDURE Modal_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "Modal", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE OutlineExpand_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "OutlineExpand")
   ENDPROC
   
   PROCEDURE OutlineExpand_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "OutlineExpand", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE OutlineVisible_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "OutlineVisible")
   ENDPROC
   
   PROCEDURE OutlineVisible_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "OutlineVisible", m.lParam)
   ENDPROC
   *----------------
   
   PROCEDURE OutlineWidth_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "OutlineWidth")
   ENDPROC
   
   PROCEDURE OutlineWidth_ASSIGN 
      LPARAMETERS nParam
      _fr_SetProperty(cPreviewOptions, "OutlineWidth", m.nParam) 
   ENDPROC
   *----------------
            
   PROCEDURE PagesInCache_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "PagesInCache")
   ENDPROC
   
   PROCEDURE PagesInCache_ASSIGN 
      LPARAMETERS nParam
      _fr_SetProperty(cPreviewOptions, "PagesInCache", m.nParam) 
   ENDPROC
   *----------------
      
   PROCEDURE ThumbnailVisible_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "ThumbnailVisible")
   ENDPROC
   
   PROCEDURE ThumbnailVisible_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "ThumbnailVisible", m.lParam) 
   ENDPROC
   *----------------
         
   PROCEDURE ShowCaptions_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "ShowCaptions")
   ENDPROC
   
   PROCEDURE ShowCaptions_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "ShowCaptions", m.lParam) 
   ENDPROC
   *----------------
   
   PROCEDURE Zoom_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "Zoom")
   ENDPROC
   
   PROCEDURE Zoom_ASSIGN 
      LPARAMETERS nParam
      _fr_SetProperty(cPreviewOptions, "Zoom", m.nParam) 
   ENDPROC
   *----------------
   
   PROCEDURE PictureCacheInFile_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "PictureCacheInFile")
   ENDPROC
   
   PROCEDURE PictureCacheInFile_ASSIGN 
      LPARAMETERS lParam
      _fr_SetProperty(cPreviewOptions, "PictureCacheInFile", m.lParam) 
   ENDPROC
   *----------------

   PROCEDURE Buttons_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "Buttons")
   ENDPROC
   
   PROCEDURE Buttons_ASSIGN 
      LPARAMETERS cParam
      _fr_SetProperty(cPreviewOptions, "Buttons", m.cParam) 
   ENDPROC
   *----------------
   
   PROCEDURE ZoomMode_ACCESS
      RETURN _fr_GetProperty(cPreviewOptions, "ZoomMode")
   ENDPROC
   
   PROCEDURE ZoomMode_ASSIGN 
      LPARAMETERS cParam
      _fr_SetProperty(cPreviewOptions, "ZoomMode", m.cParam) 
   ENDPROC
   *----------------
ENDDEFINE

